#include "buffer.h"
#include "error.h"
#include "debug_funcs.h"

int buffer_write(buffer* buf, uint8_t data, long int grow) {
    if(buf->end >= buf->size) {
        buf->size += grow;
        debug_message("buffer: reallocated buffer size to %ld bytes\n", buf->size);
        buf->data = realloc(buf->data, buf->size * sizeof(uint8_t));
    }
    buf->data[buf->end] = data;
    buf->end++;
    return ERR_OK;
}

int buffer_read(buffer* buf, uint8_t* data) {
    *data = buf->data[buf->pos];
    if(buf->pos < buf->end) buf->pos++;
    /* last byte from buffer */
    if(buf->pos == buf->end) return ERR_EOF;
    return ERR_OK;
}

buffer* buffer_create() {
    buffer* buf;
    buf = calloc(1, sizeof(buffer));
    buf->data = NULL;
    return buf;
}

void buffer_init(buffer* buf) {
    buf->size = 0;
    buf->pos = 0;
    buf->end = 0;
}

void buffer_destruct(buffer* buf) {
    if(buf->data) free(buf->data);
    free(buf);
}
