#include "container.h"
#include "error.h"
#include "file.h"
#include "debug_funcs.h"
#include "petscii_funcs.h"
#include "filetypes.h"
#include "file_funcs.h"
#include "dir_funcs.h"
#include "path_funcs.h"
#include "parse_funcs.h"
#include "string_funcs.h"
#include "container_funcs.h"

//XXX maybe add drive to parameter list? or wrap in another function to change path on drive and current partiton? -> also reopen new type then and close old type

void path_extend(char** target, char* elem) {
    /* avoid duplicate slashes */
    if(*target && elem && strlen(elem) && strlen(*target) && (*target)[strlen(*target) - 1] != '/') dyn_strcat(target, "/");
    dyn_strcat(target, elem);
}

int path_search(char** fullpath, char* chroot, char* path, char* p_glob) {
    int err = ERR_OK;
    char* temppath = NULL;
    dyn_strcpy(&temppath, path);
    err = path_change(chroot, &temppath, p_glob);
    dyn_strcpy(fullpath, chroot);
    path_extend(fullpath, temppath);
    if(temppath) free(temppath);
    return err;
}

void path_assemble(char** target, char* elem1, char* elem2) {
    dyn_strcpy(target, elem1);
    path_extend(target, elem2);
}

//! ATTENTION ! new_path must be in petscii format
//chroot und curr_path is in ascii format
int path_change(char *chroot, char **curr_path, char* new_path) {
    char* temp = NULL;
    char* glob = NULL;
    char* result_path = NULL;
    int i;
    file64* f;
    int err = ERR_OK;

    f = file_create();
    file_init(f);

    debug_message("chroot='%s'  path='%s'  new_path='%s'\n", chroot, *curr_path, new_path);
    dyn_strcpy(&result_path, *curr_path);
    dyn_strcpy(&temp, new_path);

    //we have a leading / so start from 0 and reset media
    if(temp[0] == '/') {
        //XXX but be careful, / can also be a path within a diskimage! what to do then?
        //maybe use path_media then
        debug_message("path starts with / so cutting back\n");
        result_path[0] = '\0';
    }
    //init token search
    glob = strtok(temp,"/");

    if(glob) {
        //fetch next token
        do {
            debug_message("result_path='%s' new_path='%s' glob='%s'\n", result_path, new_path, glob);

            //set up file information
            dyn_strcpy(&f->f_glob, glob);

            f->buffered = 0;
            f->f_mode = 0;
            f->f_dircmd = 0;

            path_assemble(&f->path, chroot, result_path);

            /* open container with updated path */
            container_open(f, result_path);

            /* always allow "." and ".." */
            /* search for matches */
            if(strcmp(f->f_glob, ".") == 0) {
                debug_message("doing nothing on '.'\n");
                //do nothing
            }
            else if(strcmp(f->f_glob, "..") == 0) {
                debug_message("cutting back on '..'\n");
                //cut back path if possible
                i = strlen(result_path);
                while(i && result_path[i] != '/') { result_path[i] = '\0'; i--; }
                //wipe out / as well
                result_path[i] = '\0';
            }
            /* so far, only bother about dirs on normal filesys as .d64 won't have any */
            else if(f->media == CTYPE_SYS && !file_search(f, FILETYPE_DIR)) {
                debug_message("found '%s' with glob '%s' in dir '%s', type is='%s'\n", f->ufs_name, glob, f->path, filetypes[f->filetype]);
                if(f->filetype == FILETYPE_DIR) {
                    /* extend path */
                    path_extend(&result_path, f->ufs_name);
                    debug_message("changed path to '%s'\n", result_path);
                }
            }
            else {
                debug_message("could not find glob '%s' in dir '%s'\n", glob, f->path);
                err = ERR_DIR_NOT_FOUND;
                break;
            }

            /* close container, all finished for this turn */
            container_close(f);
        }
        while((glob = strtok(NULL,"/")));
    }
    file_destruct(f);
    /* success, so copy result to curr_path */
    dyn_strcpy(curr_path, result_path);
    debug_message("resulting path is '%s'\n", *curr_path);
    free(temp);
    free(result_path);
    return err;
}
